/*
 * $Id: EmployeeHandler.java,v 1.2 2005/12/06 23:12:34 boogie Exp $
 */ 
package formdef.plugin.example.nested.business;

import java.util.*;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;


/**
 * Simulates a business tier object that handles Employee operations.
 */ 
public class EmployeeHandler {
    
    //******************************************************** Static variables

    /** The business tier instance. */
    private static EmployeeHandler instance = new EmployeeHandler();

    /** The logger instance. */
    private static Log log = LogFactory.getLog(EmployeeHandler.class);

    
    //****************************************************** Instance variables
    
    /** All available employees. */
    private Map employees;

    /** the next available employee number. */
    private int nextEmployeeNumber = 1;

    
    //******************************************* Constructor and getInstance()
    
    /** 
     * Hide the constructor. 
     */
    private EmployeeHandler() {
    }

    
    /** return the business tier instance */
    public static EmployeeHandler getInstance() {
        return instance;
    }
    
    
    //********************************************************** Public methods

    
    /** get all employees as an array*/
    public Employee[] getEmployeeArray() {
        Map employees = getEmployees();
        ArrayList list = new ArrayList(employees.size());
        Iterator iter = employees.keySet().iterator();
        while (iter.hasNext()) {
            Employee employee = (Employee) employees.get(iter.next());
            list.add(employee);
        }
        return (Employee[]) list.toArray(new Employee[list.size()]);
    }

    /** 
     * Get all employees as an ArrayList. 
     */
    public List getEmployeeList() {
        Map employees = getEmployees();
        ArrayList list = new ArrayList(employees.size());
        Iterator iter = employees.keySet().iterator();
        while (iter.hasNext()) {
            Employee employee = (Employee) employees.get(iter.next());
            list.add(employee);
        }
        return list;
    }

    /** get the employee record with the given number */
    public Employee getEmployee(int employeeNumber) {
        return (Employee) getEmployees().get(new Integer(employeeNumber));
    }

    /** add the given employee to the nested of existing ones */
    public void createEmployee(Employee employee) {
        employee.setNumber(getNextEmployeeNumber());
        getEmployees().put(new Integer(employee.getNumber()),employee);
    }

    /** update an existing employee with data from the given object */
    public void updateEmployee(Employee updates) {
        Employee employee = (Employee)
                getEmployees().get(new Integer(updates.getNumber()));

        if (employee == null) {
            throw new RuntimeException("Can't find the specified employee: "
                    + updates.getNumber());
        }
        employee.setName(updates.getName());
        employee.setAddress(updates.getAddress());
        employee.setSalary(updates.getSalary());
        employee.setBirth(updates.getBirth());
        employee.setSsNumber(updates.getSsNumber());
        employee.setEmploymentType(updates.getEmploymentType());
    }
    
    /** remove an existing employee from the nested */
    public void removeEmployee(int employeeNumber) {
        employees.remove(new Integer(employeeNumber));
    }
    
    
    //******************************************************** Internal methods

    
    /** get all employees, or create a fresh set if there isn't any */
    private Map getEmployees() {
        if ((employees == null) || (employees.size() <= 0)) {
            try {
                employees = createEmployees();
            } catch (ParseException e) {
                log.error("Unexpected exception while creating employees.", e);
                throw new RuntimeException(e);
            }
        }
        return employees;
    }

    /** create some employees to play with */
    private Map createEmployees()
            throws ParseException {
        if (log.isDebugEnabled()) {
            log.debug("Creating classes for testing...");
        }
        Map employees = new LinkedHashMap();
        ArrayList phones = null;

        SimpleDateFormat sdf = new SimpleDateFormat("MM/dd/yyyy");
        Employee employee = new Employee();
        employee.setNumber(getNextEmployeeNumber());
        employee.setName("John Doe");
        employee.setAddress(new Address("123 Main St.","Ellisville","MO",63011));
        employee.setBirth(sdf.parse("12/25/1970"));
        employee.setEmploymentType(EmploymentType.TYPE_FULL_TIME);
        employee.setSsNumber(123456780);
        employee.setSalary(125000);
        phones = new ArrayList();
        phones.add("555-0130");
        phones.add("555-0131");
        phones.add("555-0132");
        employee.setPhones(phones);
        log.debug("Adding " + employee);
        employees.put(new Integer(employee.getNumber()),employee);

        employee = new Employee();
        employee.setNumber(getNextEmployeeNumber());
        employee.setName("Jane Doe");
        employee.setAddress(new Address("123 Main St.","Gresham","OR",97030));
        employee.setBirth(sdf.parse("05/01/1978"));
        employee.setEmploymentType(EmploymentType.TYPE_FULL_TIME);
        employee.setSsNumber(123456782);
        employee.setSalary(45000);
        phones = new ArrayList();
        phones.add("555-0100");
        phones.add("555-0155");
        employee.setPhones(phones);
        log.debug("Adding " + employee);
        employees.put(new Integer(employee.getNumber()),employee);

        employee = new Employee();
        employee.setNumber(getNextEmployeeNumber());
        employee.setName("Mark Johnson");
        employee.setAddress(new Address("6 Maple Rd.","Cedar Bluffs","NE",68015));
        employee.setBirth(sdf.parse("08/19/1975"));
        employee.setEmploymentType(EmploymentType.TYPE_PART_TIME);
        employee.setSsNumber(100340000);
        employee.setSalary(20000);
        phones = new ArrayList();
        phones.add("555-1202");
        employee.setPhones(phones);
        log.debug("Adding " + employee);
        employees.put(new Integer(employee.getNumber()),employee);

        employee = new Employee();
        employee.setNumber(getNextEmployeeNumber());
        employee.setName("Pam Rutherford");
        employee.setAddress(new Address("17 Oak Dr.","Spring Lake","NC",28390));
        employee.setBirth(sdf.parse("02/13/1969"));
        employee.setEmploymentType(EmploymentType.TYPE_FULL_TIME);
        employee.setSsNumber(321654987);
        employee.setSalary(1200000);
        phones = new ArrayList();
        phones.add("555-0129");
        phones.add("555-0100");
        phones.add("555-0200");
        employee.setPhones(phones);
        log.debug("Adding " + employee);
        employees.put(new Integer(employee.getNumber()),employee);

        return employees;
    }

    /** get the next available employee number */
    private int getNextEmployeeNumber() {
        //int result = 1;
        //Iterator iter = employees.keySet().iterator();
        //while (iter.hasNext()) {
        //    Employee employee = (Employee) employees.get(iter.next());
        //    if ((employee != null) && (employee.getNumber() >= result)) {
        //        result = employee.getNumber() + 1;
        //    }
        //}
        //return result;
        return nextEmployeeNumber++;
    }

}
