/*
 * $Id: TestPropertyMappingConfig.java,v 1.5 2005/12/17 17:21:37 boogie Exp $
 */ 
package formdef.plugin.config;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.apache.struts.config.FormPropertyConfig;

import formdef.plugin.sample.SampleIndexedBeanA;
import formdef.plugin.FormMapping;
import formdef.plugin.PropertyMapping;
import formdef.plugin.digester.DefaultDigester;
import formdef.plugin.conversion.NumberConverter;
import formdef.plugin.conversion.FormCollectionConverter;

import java.io.InputStream;
import java.io.BufferedInputStream;

/**
 * Various tests for the PropertyMappingConfig class.
 */
public class TestPropertyMappingConfig extends TestCase {
    
    //*****************************
    // the usual suspects

    public TestPropertyMappingConfig(String s) {
        super(s);
    }

    public static TestSuite getSuite() {
        return new TestSuite(TestPropertyMappingConfig.class);
    }

    public static void main(String[] args) {
        junit.textui.TestRunner runner = new junit.textui.TestRunner();
        runner.doRun(TestPropertyMappingConfig.getSuite());
    }
    
    //*****************************
    
    /** The FormDefConfig that will hold the configured forms */
    protected FormDefConfig formDefConfig = null;

    
    public void setUp()
            throws Exception {
        super.setUp();

        String filename = "formdef/plugin/config/propertymappingconfig-nestedcollections.xml";
        ClassLoader classLoader = this.getClass().getClassLoader();
        InputStream is = classLoader.getResourceAsStream(filename);
        formDefConfig = new FormDefConfig();
        DefaultDigester.readConfiguration(formDefConfig, new BufferedInputStream(is));
        is.close();
    }
    
    /**
     * Test that we get the correct converter for an int field.
     */ 
    public void testGeneratePropertyInt()
            throws Exception {
        
        FormMapping form = 
                new FormMapping("sampleIndexedBeanA1", SampleIndexedBeanA.class);
        
        // create the object we'll be testing
        PropertyMappingConfig propConfig = new PropertyMappingConfig();
        propConfig.setName("int1");
        propConfig.setGetter("getInt1");
        propConfig.setSetter("setInt1");
        
        PropertyMapping result = propConfig.generateProperty(form, formDefConfig);
        
        assertNotNull("Results can't be null", result);
        assertTrue("Invalid converter", 
                result.getConverter() instanceof NumberConverter);
        
        assertEquals("Wrong property name", "int1", result.getName());
        assertNull("Invalid conversion parameter", result.getConversionParam());
        assertEquals("Type must be int", Integer.TYPE, result.getType());
        assertEquals("FormFieldType must be String", String.class, result.getFormFieldType());
    }
    
    /**
     * Check that we handle array properties correctly:<br/>
     *  <ul><li>We don't get a null result</li>
     *      <li>The type on the resulting mapping is correct</li>
     *      <li>We don't assign a converter</li>
     *  </ul>
     */ 
    public void testGeneratePropertyArray()
            throws Exception {
        
        FormMapping form = 
                new FormMapping("sampleIndexedBeanA1", SampleIndexedBeanA.class);
        
        // create the object we'll be testing
        PropertyMappingConfig propConfig = new PropertyMappingConfig();
        propConfig.setName("sampleBean1s");
        propConfig.setGetter("getSampleBean1s");
        propConfig.setSetter("setSampleBean1s");
        propConfig.setType("formdef.plugin.sample.SampleBean1[]");
        
        PropertyMapping result = propConfig.generateProperty(form, formDefConfig);
        
        assertNotNull("Results can't be null", result);
        assertEquals("Wrong property name", "sampleBean1s", result.getName());
        assertNull("Invalid conversion parameter", result.getConversionParam());
        assertNull("Converter must be null", result.getConverter());

        FormPropertyConfig formPropertyConfig = new FormPropertyConfig();
        formPropertyConfig.setType(propConfig.getType());
        assertEquals("Types do not match",
                formPropertyConfig.getTypeClass(), result.getType());
        assertEquals("FormFieldType in incorrect",
                formPropertyConfig.getTypeClass(), result.getFormFieldType());
    }
    
    
    public void testFullyUserConfiguredFormCollectionConverter() 
            throws Exception {
        FormMapping form = formDefConfig.getForm("nestingForm3");
        PropertyMapping property = (PropertyMapping) 
                form.getProperties().get("nestingBean2");
        
        Object object = property.getConverter();
        
        assertTrue("Incorrect converter type",
                object instanceof FormCollectionConverter);
        
        assertEquals("Incorrect conversion param", "nestingForm2",
                property.getConversionParam());
        assertEquals("Incorrect form collection type",
                "java.util.ArrayList",
                property.getProperties().getProperty(
                        FormCollectionConverter.PROP_FORM_COLLECTION_TYPE));
        assertEquals("Incorrect bean collection type",
                "java.util.ArrayList",
                property.getProperties().getProperty(
                        FormCollectionConverter.PROP_BEAN_COLLECTION_TYPE));
    }
    
    
    public void testPartiallyUserConfiguredFormCollectionConverter() 
            throws Exception {
        FormMapping form = formDefConfig.getForm("nestingForm3b");
        PropertyMapping property = (PropertyMapping) 
                form.getProperties().get("nestingBean2");
        
        Object object = property.getConverter();
        
        assertTrue("Incorrect converter type",
                object instanceof FormCollectionConverter);
        
        assertEquals("Incorrect conversion param", "nestingForm2",
                property.getConversionParam());
        assertEquals("Incorrect form collection type",
                "java.util.ArrayList",
                property.getProperties().getProperty(
                        FormCollectionConverter.PROP_FORM_COLLECTION_TYPE));
        assertEquals("Incorrect bean collection type",
                "java.util.ArrayList",
                property.getProperties().getProperty(
                        FormCollectionConverter.PROP_BEAN_COLLECTION_TYPE));
    }
    
    
    public void testAutoConfiguredFormCollectionConverter() 
            throws Exception {
        FormMapping form = formDefConfig.getForm("nestingForm4");
        PropertyMapping property = (PropertyMapping) 
                form.getProperties().get("nestingBean2");
        
        Object object = property.getConverter();
        
        assertTrue("Incorrect converter type",
                object instanceof FormCollectionConverter);
        
        assertEquals("Incorrect conversion param", "nestingForm2",
                property.getConversionParam());
        assertEquals("Incorrect form collection type",
                "java.util.ArrayList",
                property.getProperties().getProperty(
                        FormCollectionConverter.PROP_FORM_COLLECTION_TYPE));
        assertEquals("Incorrect bean collection type",
                "java.util.ArrayList",
                property.getProperties().getProperty(
                        FormCollectionConverter.PROP_BEAN_COLLECTION_TYPE));
    }
}
